#property copyright "Scriptong"
#property link "scriptong@mail.ru"

#property indicator_separate_window
#property indicator_buffers 5
#property indicator_color1 Blue
#property indicator_color2 Red
#property indicator_color3 C'79, 123, 153'
#property indicator_color4 C'134, 53, 53'
#property indicator_color5 DarkGray
               
#property indicator_width3 2
#property indicator_width4 2
#property indicator_width5 2

//   
extern int       i_indBarsCount        = 5000;

//  
double g_bullPower[];
double g_bearPower[];
double g_bullsPrevalence[];
double g_bearsPrevalence[];
double g_noPrevalence[];

//    
bool g_activate,                                                                                   //    
     g_init;                                                                                       //          ..
                                                                                                   // .. 
                                                                                                   
double g_tickSize;
                                                                                                   
     
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Custom indicator initialization function                                                                                                                                                          |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int init()
{
   g_activate = false;                                                                             //   
   g_init = true;
   
   if (!IsTuningParametersCorrect())                                                               //      -   
      return (-1);                                 
           
   if (!BuffersBind())
      return (-1);                                                   
           
   g_activate = true;                                                                              //   
   return(0);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                          |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
bool IsTuningParametersCorrect()
{
   string name = WindowExpertName();

   if (Period() > PERIOD_H4)
   {
      Print(name, ":      ,  H4.");
      return (false);
   }

   int period = Period();
   if (period == 0)
   {
      Alert(name, ":    -  0 .  .");
      return (false);
   }
   
   if (Point == 0)
   {
      Alert(name, ":    -    . ",
                  " .");
      return (false);
   }
   
   g_tickSize = MarketInfo(Symbol(), MODE_TICKSIZE);

   
   return (true);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Custom indicator deinitialization function                                                                                                                                                        |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int deinit()
{
   return(0);
}
//+------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                     |
//+------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
double NP(double value)
{
   if (g_tickSize == 0)
      return (0);
      
   return (MathRound(value / g_tickSize) * g_tickSize);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                              |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
bool BuffersBind()
{
   string name = WindowExpertName();

   //     
   if (!SetIndexBuffer(0, g_bullPower)             ||
       !SetIndexBuffer(1, g_bearPower)             ||
       !SetIndexBuffer(2, g_bullsPrevalence)       ||
       !SetIndexBuffer(3, g_bearsPrevalence)       ||
       !SetIndexBuffer(4, g_noPrevalence))
   {
      Alert(name, ":      .  ", GetLastError());
      return (false);
   }

   //    
   for (int i = 0; i < 2; i++)   
      SetIndexStyle(i, DRAW_LINE);
   for (i = 2; i < 5; i++)   
      SetIndexStyle(i, DRAW_HISTOGRAM);

   SetIndexLabel(0, "Bull power");
   SetIndexLabel(1, "Bear power");
   SetIndexLabel(2, "Bulls prevalence");
   SetIndexLabel(3, "Bears prevalence");
   SetIndexLabel(4, "No prevalence");
   
   return (true);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                     |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void BuffersInitialize()
{
   ArrayInitialize(g_bullPower, EMPTY_VALUE);
   ArrayInitialize(g_bearPower, EMPTY_VALUE);
   ArrayInitialize(g_bullsPrevalence, EMPTY_VALUE);
   ArrayInitialize(g_bearsPrevalence, EMPTY_VALUE);
   ArrayInitialize(g_noPrevalence, EMPTY_VALUE);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|   ,                                                                                                                                 |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int GetRecalcIndex(int& total)
{
   //      
   static int lastBarsCnt;                                                
   if (g_init)                                     
   {                                               
      lastBarsCnt = 0;
      g_init = false;
   }
   total = Bars - 2;                                                                               //    ,       
                                                   
   //          ?
   if (i_indBarsCount > 0 && i_indBarsCount < total)
      total = i_indBarsCount;                      
                                                   
   //       , . .          ,     ,      
   if (lastBarsCnt < Bars - 1)                     
   {                                               
      lastBarsCnt = Bars;
      BuffersInitialize();
      return (total);
   }
   
   //   .            ,    
   int newBarsCnt = Bars - lastBarsCnt;
   lastBarsCnt = Bars;
   return (newBarsCnt);                            
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
double GetTypicalPrice(int index)
{
   return (NP((High[index] + Low[index] + Close[index]) / 3));
}
//+------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|    ,  ?                                                                                                                                                      |
//+------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
bool IsFirstMoreThanSecond(double first, double second)
{
   return (first - second > Point / 10);
}
//+------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|    ?                                                                                                                                                          |
//+------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
bool IsValuesEquals(double first, double second)
{
   return (MathAbs(first - second) < Point / 10);
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                       |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void ProcessOneCandle(int index)
{
   //        
   if (TimeDayOfYear(Time[index]) != TimeDayOfYear(Time[index + 1]))
   {
      g_bullPower[index] = 0;
      g_bearPower[index] = 0;
      return;
   }

   //       
   double priceCur = GetTypicalPrice(index);
   double pricePrev = GetTypicalPrice(index + 1);
   
   //        -  
   g_bullPower[index] = g_bullPower[index + 1];
   g_bearPower[index] = g_bearPower[index + 1];
   if (IsValuesEquals(priceCur, pricePrev))
      return;
      
   //    -    
   if (IsFirstMoreThanSecond(priceCur, pricePrev))
   {
      g_bullPower[index] = g_bullPower[index + 1] + Volume[index];
      return;
   }

   //    -    
   g_bearPower[index] = g_bearPower[index + 1] + Volume[index];
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                          |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void ShowPrevalencePower(int index)
{
   g_noPrevalence[index] = EMPTY_VALUE;
   g_bearsPrevalence[index] = EMPTY_VALUE;
   g_bullsPrevalence[index] = EMPTY_VALUE;

   //    
   if (g_bearPower[index] == g_bullPower[index])
   {
      g_noPrevalence[index] = g_bearPower[index];
      return;
   }

   //    
   if (g_bearPower[index] > g_bullPower[index])
   {
      g_bearsPrevalence[index] = g_bearPower[index];
      return;
   }
   
   //    
   g_bullsPrevalence[index] = g_bullPower[index];
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                                                                        |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
void ShowIndicatorData(int limit, int total)
{
   for (int i = limit; i >= 0; i--)
   {
      ProcessOneCandle(i);
      ShowPrevalencePower(i);
   }
}
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
//| Custom indicator iteration function                                                                                                                                                               |
//+---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
int start()
{
   if (!g_activate)                                                                                //     ,     
      return (0);                                 
                                                   
   int total;   
   int limit = GetRecalcIndex(total);                                                              //     ?

   ShowIndicatorData(limit, total);                                                                //   
   
   return(0);
}